body {
    background:                 #FFFFFF url('img/pinstripes.png') repeat scroll 0 0;
    color:                      #000000;
    overflow:                   hidden;
    margin:                     0px 0px 0px 0px;
    padding:                    40px 0px 0px 0px;
    font-family:                Helvetica;
/* ensure that we don't get the default callout following a long touch on an element, which is a default behavior in Mobile Safari */
    -webkit-touch-callout:      none;
/* disable the Mobile Safari default behavior to adjust font size automatically to improve readability */
    -webkit-text-size-adjust:   none;
    width:                      320px;
    min-height:                 380px;
}

body.landscape {
    width:                      480px;
    min-height:                 230px;
}
