#ifndef GALLERYSETTINGS_H
#define GALLERYSETTINGS_H

#include "mythtv/settings.h"
#include "mythtv/mythcontext.h"

class GallerySettings : public ConfigurationWizard
{
  public:  
    GallerySettings();
};

#endif
