void strings_null() {
    ThemeUI::tr("Please Wait ...");
    ThemeUI::tr("Track Information");
    ThemeUI::tr("Compilation:");
    ThemeUI::tr("Comp. Artist:");
    ThemeUI::tr("Artist:");
    ThemeUI::tr("Album:");
    ThemeUI::tr("Title:");
    ThemeUI::tr("Track No.:");
    ThemeUI::tr("Genre:");
    ThemeUI::tr("Year:");
    ThemeUI::tr("Rating:");
    ThemeUI::tr("Play Count:");
    ThemeUI::tr("Last Play:");
    ThemeUI::tr("Filename:");
    ThemeUI::tr("Album Art");
    ThemeUI::tr("No Album Art Found");
    ThemeUI::tr("Type:");
    ThemeUI::tr("Import Coverart");
    ThemeUI::tr("Browse cover images on your system.");
    ThemeUI::tr("File:");
    ThemeUI::tr("Destination:");
    ThemeUI::tr("Image Type:");
    ThemeUI::tr("Previous");
    ThemeUI::tr("Next");
    ThemeUI::tr("Copy");
    ThemeUI::tr("Done");
    ThemeUI::tr("Already in database");
    ThemeUI::tr("Import Music");
    ThemeUI::tr("Import local music files.");
    ThemeUI::tr("Location:");
    ThemeUI::tr("Track is part of a compilation");
    ThemeUI::tr("Click to browse directories...");
    ThemeUI::tr("Search");
    ThemeUI::tr("Choose Coverart");
    ThemeUI::tr("Add");
    ThemeUI::tr("Play");
    ThemeUI::tr("Next New");
    ThemeUI::tr("Add All New");
    ThemeUI::tr("Rip A CD");
    ThemeUI::tr("Input metadata for the CD you have inserted.");
    ThemeUI::tr("Quality:");
    ThemeUI::tr("Album Name:");
    ThemeUI::tr("Album Year:");
    ThemeUI::tr("Album is a compilation");
    ThemeUI::tr("Click here to search existing artists...");
    ThemeUI::tr("Click here to search existing albums...");
    ThemeUI::tr("Click here to search existing genres...");
    ThemeUI::tr("Switch Titles");
    ThemeUI::tr("Scan CD");
    ThemeUI::tr("Rip CD");
    ThemeUI::tr("Metadata");
    ThemeUI::tr("Cover Art");
    ThemeUI::tr("Multi-Artist:");
    ThemeUI::tr("Track");
    ThemeUI::tr("Title");
    ThemeUI::tr("Artist");
    ThemeUI::tr("Length");
    ThemeUI::tr("Encoding Track");
    ThemeUI::tr("Edit Playlist");
    ThemeUI::tr("Play Music");
    ThemeUI::tr("Please Wait...");
    ThemeUI::tr("Edit Metadata");
    ThemeUI::tr("Edit AlbumArt");
    ThemeUI::tr("Ripping CD");
    ThemeUI::tr("Importing CD");
    ThemeUI::tr("Directory Finder");
    ThemeUI::tr("Import Cover Art");
    ThemeUI::tr("/some/file/goes/here/test.mp3");
    ThemeUI::tr("123 of 345");
    ThemeUI::tr("All ready in database");
    ThemeUI::tr("%1%");
    ThemeUI::tr("General Settings");
    ThemeUI::tr("Music folders and general options");
    ThemeUI::tr("Player Settings");
    ThemeUI::tr("Music playback options");
    ThemeUI::tr("Ripper Settings");
    ThemeUI::tr("CD copying options");
    ThemeUI::tr("Select Music");
    ThemeUI::tr("Import CD");
    ThemeUI::tr("Import Files");
    ThemeUI::tr("Scan for New Music");
    ThemeUI::tr("Eject media");
    ThemeUI::tr("Music Settings");
    ThemeUI::tr("Configure playback and CD ripping");
}
