Body {
	border: 0;
	margin: 0;
	padding: 0;
	font-size: 10px;
	font-family: Arial, Helvetica, sans-serif;
	background-color: #002650;
	color: #DEDEDE;
}

/* make transparent png's render properly in IE */
/*
	Disabled because it conflicts with recording preview thumbnails
	It can be enabled on individual images by:
		style="behavior: url('/pngbehavior.htc');"

	img { behavior: url("themes/Default/pngbehavior.htc"); }
*/

A:link    { color: #E0E0FF; text-decoration: none; }
A:active  { color: #990033; text-decoration: none; }
A:visited { color: #E0E0FF; text-decoration: none; }
A:hover   { color: #F0F000; text-decoration: underline; }

/* For some reason, the main Body element doesn't always work for text inside of tables */
TD {
	font-size: 10px;
	font-family: Arial, Helvetica, sans-serif;
}

/* font size classes */
.tiny  { font-size: 9px; }
.small { font-size: 10px; }
.large { font-size: 12px; }
.huge  { font-size: 24px; }

/* a class for the menu across the top of the page as well as menu headers throughout the page */
.menu {	background-color: #265990 }
.menu_border_t { border-top:    2px solid #9090B0 }
.menu_border_b { border-bottom: 2px solid #9090B0 }
.menu_border_l { border-left:   2px solid #9090B0 }
.menu_border_r { border-right:  2px solid #9090B0 }

/* a class for commands and other user input boxes */
.command { background-color: #1040A0 }
.command_border_t { border-top:    2px solid #9090B0 }
.command_border_b { border-bottom: 2px solid #9090B0 }
.command_border_l { border-left:   2px solid #9090B0 }
.command_border_r { border-right:  2px solid #9090B0 }

/* some classes for the list display */
.list {	background-color: #002650 }
.list_border_t { border-top:    1px solid #9090B0 }
.list_border_b { border-bottom: 1px solid #9090B0 }
.list_border_l { border-left:   1px solid #9090B0 }
.list_border_r { border-right:  1px solid #9090B0 }

/* a class for the the cells containing settings in the various settings files */
.settings {	background-color: #506080 }

/* a class for the the cells containing recored programs in recorded_programs.php */
.recorded {	background-color: #506080 }

/* a class for the the cells containing recored programs in recorded_programs.php */
.scheduled	 {	background-color: #506090 }
.conflict	 {	background-color: #806050 }
.duplicate   {	background-color: #304050 }
.deactivated {	background-color: #508060 }

/* A special class for submit buttons */
.submit {
	font-weight: bold;
    color: #001060;
    background-color: #C0C0D0;
	font-size: 13px;
	height: 22px;
}
.submit:hover   { text-decoration: underline; }

/* A special class for radio buttons because some browsers render them weird */
.radio { height: 14px }

/* Default styles for form fields */
Select {
    font-family: Arial, Helvetica, sans-serif;
    color: #001060;
    background-color: #C0C0D0;
    font-size: 12px;
    height: 18px;
}

Input {
    font-family: Arial, Helvetica, sans-serif;
    color: #001060;
    background-color: #C0C0D0;
    font-size: 14px;
    height: 22px;
}

TextArea {
    font-family: terminal, courier, courier-new;
    color: #001060;
    background-color: #C0C0D0;
    font-size: 12px;
}

/* A special class for form stuff, to avoid those nasty extra linefeeds in IE </form> tags */
.form { display: inline; }

/* A special class for error stuff */
.error {
    color: #F03030;
    background-color: #360000;
    border-color: #F03030;
    border: thin groove #F03030;
}

/* for mouseover popup menus */
.hidden {
	position: absolute;
	visibility: hidden;
	z-index: 99;
	top: 0px;
	left: 0px;
}

/*

	The following subclasses are used for list displays

*/

.record_weekly      { border: 1px solid #00D000; }
.record_timeslot    { border: 1px solid #00D000; }
.record_once	    { border: 1px solid #00D000; }
.record_channel	    { border: 1px solid #00D000; }
.record_always	    { border: 1px solid #00D000; }
.will_record	    { border: 1px solid #00D000; }
.record_conflicting { border: 1px solid #FF0000; }
.record_duplicate   { border: 2px dashed #90B0A0; }
.record_suppressed  { border: 2px dotted #60B0B0; }

.cat_Action         { background-color: #906020 }
.cat_Adult          { background-color: #702020 }
.cat_Animals        { background-color: #609060 }
.cat_Art_Music      { background-color: #801060 }
.cat_Business       { background-color: #703010 }
.cat_Children       { background-color: #B00010 }
.cat_Comedy         { background-color: #006080 }
.cat_Crime_Mystery  { background-color: #105050 }
.cat_Documentary    { background-color: #504020 }
.cat_Drama          { background-color: #400060 }
.cat_Educational    { background-color: #606060 }
.cat_Food           { background-color: #208040 }
.cat_Game           { background-color: #701010 }
.cat_Health_Medical { background-color: #806060 }
.cat_History        { background-color: #807020 }
.cat_HowTo          { background-color: #A0A000 }
.cat_Horror         { background-color: #101040 }
.cat_Misc           { background-color: #403060 }
.cat_News           { background-color: #606040 }
.cat_Reality        { background-color: #304040 }
.cat_Romance        { background-color: #A02050 }
.cat_Science_Nature { background-color: #007010 }
.cat_SciFi_Fantasy  { background-color: #606090 }
.cat_Shopping       { background-color: #204020 }
.cat_Soaps          { background-color: #508080 }
.cat_Spiritual      { background-color: #804080 }
.cat_Sports         { background-color: #20B060 }
.cat_Talk           { background-color: #203040 }
.cat_Travel         { background-color: #2060B0 }
.cat_War            { background-color: #B06050 }
.cat_Western        { background-color: #806040 }
.cat_Unknown        { background-color: #303030 }

.cat_movie          { background-color: #809090 !important; }
.cat_series         { }
.cat_tvshow         { }

/* deprecated show categories that need to be integreted into the above list via regex, or have new categories created for them */
.tv_Cine           { background-color: Blue }
.tv_Classic        { background-color: gray }
.tv_Collectibles   { background-color: SlateGrey }
.tv_Magazine       { background-color: MediumTurquoise }
.tv_Movies         { background-color: Blue }
.tv_Serie          { background-color: DarkOrchid }
.tv_SHOW           { background-color: sienna }
.tv_Tiempo         { background-color: #33ff99 }
