#content_wrapper {
    width:  660px;
    margin: 0px auto
}

h1 {
    font-size:      28px;
    font-weight:    900;
    color:          #777777;
    letter-spacing: 0.5em;
    margin-bottom:  30px;
    width:          650px;
    text-align:     center;
}
h2 {
    font-size:      18px;
    font-weight:    800;
    color:          #7c4;
    border:         none;
    letter-spacing: 0.3em;
    padding:        0px;
    margin-bottom:  10px;
    margin-top:     0px;
}
hr {
    display:none;
}
div.content {
    width:              650px;
    /*border-top:         1px  solid #88a;
    border-right:       1px  solid #88a;
    border-bottom:      1px  solid #88a;
    border-left:        10px solid #88a;*/
    border:             1px solid #333;
    padding:            10px;
    margin-bottom:      30px;
    //-moz-border-radius: 8px 0px 0px 8px;
}
div.schedule a {
    position:        relative;
    display:         block;
    color:           000000;
    text-decoration: none;
    padding:         .2em .8em;
    border:          thin solid #dddddd;
    width:           340px;
}
div.schedule a span {
    display:none;
}
div.schedule a:hover {
    background-color: #fafafa;
    /*border-top:       thin solid #668;
    border-bottom:    thin solid #668;
    border-left:      thin solid #668;*/
    cursor:           default;
}
div.schedule a:hover span {
    display:          block;
    position:         absolute;
    background-color: #fafafa;
    color:            #000000;
    left:             358px;
    margin-top:       -20px;
    width:            280px;
    padding:          5px;
    border:           thin dashed black;
}
div.diskstatus {
    width:  325px;
    height: 7em;
    float:  left;
}
div.loadstatus {
    width:  325px;
    height: 7em;
    float:  right;
}
.jobfinished { color: white; }
.jobaborted  { color: #ff2222;  }
.joberrored  { color: #ff4444;  }
.jobrunning  { color: #00ff00;  }

