#content_wrapper {
    padding-left: 	1em;
    padding-right: 	1em;
    padding-bottom: 	1em;
    margin: 		auto;
}

h2 {
    text-align: 	center;
}

#count_div {
    float: 		left;
    padding-left: 	5em;
    padding-top: 	1em;
}
#query_time_div {
    float: 		right;
    padding-right: 	5em;
    padding-top: 	1em;
}

/* The general stats box */
#general_stats dl {
    font-size:  	9pt;
    max-width:  	35em;
    margin:     	1em auto;
    padding:    	.2em .5em .5em .5em;
    border:     	1px dotted #9090B0;
}

#general_stats dt {
    clear:       	left;
    float:       	left;
    width:       	15em;
    padding-top: 	3px;
    text-align:  	right;
}

#general_stats dd {
    padding-top: 	3px;
    margin-left: 	15.5em;
}

#stats {
    border:             1px solid #9090B0;
    border-collapse:    collapse;
    margin:		auto;
}

.caption {
    border:             1px dotted #9090B0;
    font-weight:        bold;
    text-align:		center;
}

.center {
    text-align:		center;
}

#stats td {
    border:             1px dotted #9090B0;
    padding:            0px .5em;
}

#stats th {
    border:             1px solid  #9090B0;
    border-right:       1px dotted #9090B0;
    border-left:        1px dotted #9090B0;
    padding:            0px .5em;
}
