void strings_null() {
    ThemeUI::tr("Movie Times");
    ThemeUI::tr("Rated %1");
    ThemeUI::tr("Rating:");
    ThemeUI::tr("Running Time:");
    ThemeUI::tr("Show Times:");
    ThemeUI::tr("Theater name");
}
