void strings_null() {
    ThemeUI::tr("Games");
    ThemeUI::tr("Cancel");
    ThemeUI::tr("Play");
    ThemeUI::tr("Edit Metadata");
    ThemeUI::tr("Modify your game's metadata.");
    ThemeUI::tr("Title:");
    ThemeUI::tr("Genre:");
    ThemeUI::tr("Year:");
    ThemeUI::tr("Country:");
    ThemeUI::tr("Publisher:");
    ThemeUI::tr("Plot:");
    ThemeUI::tr("Cover Art:");
    ThemeUI::tr("Screenshot:");
    ThemeUI::tr("Fanart:");
    ThemeUI::tr("Game is a Favorite");
    ThemeUI::tr("Click here to search existing coverart...");
    ThemeUI::tr("Click here to search existing screenshots...");
    ThemeUI::tr("Click here to search existing fanart...");
    ThemeUI::tr("Done");
    ThemeUI::tr("System:");
    ThemeUI::tr("Favorite:");
    ThemeUI::tr("No");
    ThemeUI::tr("Yes");
    ThemeUI::tr("ROM Name:");
    ThemeUI::tr("ROM Path:");
    ThemeUI::tr("/path/to/the/thing.jpg");
    ThemeUI::tr("Video Manager");
    ThemeUI::tr("Game Information");
    ThemeUI::tr("Game:");
    ThemeUI::tr("Type:");
    ThemeUI::tr("Filename:");
    ThemeUI::tr("Rompath:");
    ThemeUI::tr("System(s):");
    ThemeUI::tr("CRC:");
    ThemeUI::tr("Edit Game Information");
    ThemeUI::tr("Game Name:");
    ThemeUI::tr("Description:");
    ThemeUI::tr("Boxart:");
    ThemeUI::tr("No Plot.");
    ThemeUI::tr("General Settings");
    ThemeUI::tr("Game folders and general options");
    ThemeUI::tr("Game Players");
    ThemeUI::tr("Configure an emulator");
    ThemeUI::tr("Scan for Games");
    ThemeUI::tr("Add games to your library");
    ThemeUI::tr("Clear Game Data");
    ThemeUI::tr("Empty your game library");
}
