#backend_logs {
    margin:             10px auto;
    border:             1px solid #48f;
    border-right:       2px solid #48f;
    border-collapse:    collapse;
    color:              white;
    width:              99%;
}

#backend_logs td {
    padding:        2px;
    border:         1px solid #48f;
    border-right:   1px solid #04a;
}
#backend_logs th {
    padding:        2px 5px;
    border:         1px solid #48f;
    border-right:   1px solid #04a;
    text-align:     left;
}

#backend_logs tr.header {
    background-color:   #134;
    color:              #ffb;
}

#backend_logs tr.odd {
    background-color:   #475;
}

#backend_logs tr.even {
    background-color:   #355;
}
