#program_content {
    margin: 10px auto;
    width: 800px;
    //max-width: 1000px;
    font-size: 9pt;
}
#program_content td {
    vertical-align: top;
    padding:        0px 10px;
}

#program_info {
    min-width: 350px;
    max-width: 500px;
    margin:    0px;
}

#program_header {
    margin-bottom: 15px;
}

#channel_info {
    float:      left;
    width:      110px;
    text-align: center;
    padding:    5px;
}
#channel_info span.preferred {
    position:    relative;
    top:         -4px;
    margin-left: 5px;
    font-size:   32px;
}

#program_title {
    margin-left: 135px;
}
#program_title h1 {
    padding: 0px;
    margin:  0px;
}
#program_time {
    white-space: nowrap;
}
#external_searches {
    white-space: nowrap;
}


#program_details {
}
#program_details dl, #program_extra_details dl {
    margin: 5px 0px;
}
#program_details dl dt, #program_extra_details dl dt {
    clear:       left;
    float:       left;
    width:       8em;
    padding-top: 3px;
    text-align:  right;
    color:       #ffe;
    white-space: nowrap;
}
#program_details dl dd, #program_extra_details dl dd {
    padding-top: 3px;
    margin-left: 8.5em;
}

#program_extra_details {
}

#conflicting_shows {
    margin:      20px auto 0px auto;
    padding:     10px;
    border:      1px solid #000;
    text-align:  center !important;
}
#conflicting_shows table {
    margin: auto;
}
#conflicting_shows table td {
    padding: 3px;
}

#local_links {
    width:       20em;
    margin:      20px auto 0px auto;
    padding:     10px;
    border:      1px solid #000;
    text-align:  center !important;
    white-space: nowrap;
}
#local_links a {
    display: block;
    padding: 5px;
}



#recording_info {
    padding:    10px;
}
#recording_info h3 {
    margin: 0px;
}
#recording_info ul {
    padding-left: 1em;
}
#recording_info li {
    list-style-type: none;
    white-space:     nowrap;
}

#schedule_override {
    width:  29em;
    margin: 0px auto;
}
#schedule_options {
    width:  29em;
    margin: 0px auto;
}

#advanced_options {
    width:  29em;
    margin: 0px auto;
}
#advanced_options dl {
    margin: 5px 10px;
}
#advanced_options dl dt {
    clear:         left;
    float:         left;
    width:         13em;
    margin-bottom: 0.25em;
    text-align:    right;
    white-space:   nowrap;
}
#advanced_options dl dd {
    margin-bottom: 0.25em;
    margin-left:   13.5em;
}
#advanced_options input {
}
#advanced_options select {
    width: 14em;
}

