/*
 *
 * Styles used in /tv/list
 *
/*/

.right_arrow {
    float: right;
}
.left_arrow {
    float: left;
}

#list_head {
    width:              80%;
    margin:             1em auto;
}

#x_current_time {
    font-size:          150%;
    margin-top:         .25em;
    float:              left;
}
#x_current_time img {
    margin-top:         -.25em;
    float:              left;
    margin-right:       .5em;
}

#x-jumpto {
    float:              right;
}
#x-jumpto td {
    padding:            .35em 1em;
    vertical-align:     middle;
    white-space:        nowrap;
}
#x-jumpto td.x-jumpto {
    font-size:          110%;
    font-weight:        bold;
    border-right:       1px solid #304943;
}
#x-jumpto td.x-hour {
    border-right:       1px solid #304943;
}
#x-jumpto td.x-hour select {
    text-align:         right;
}
#x-jumpto .x-day select {
    text-align:         center;
}

#x-jumpto a img {
    vertical-align:     middle;
    height:             16px;
    width:              16px;
    margin-top:         -2px;
}
#x-jumpto a {
    height:             20px;
    width:              20px;
    padding:            2px;
    border:             1px solid #7b8;
    background-color:   #263;
}
#x-jumpto a:hover {
    border:             1px solid #9da;
    background-color:   #485;
}
