/*

    The following subclasses are used for list displays

*/

/* a class for the the cells containing recorded programs in recorded_programs.php */
.scheduled   { background-color: #506090; }
.conflict    { background-color: #806050; }
.duplicate   { background-color: #304050; }
.deactivated { background-color: #508060; }

.will_record              { border: 1px solid  #00F000; }
.record_weekly            { border: 1px solid  #00F000; }
.record_timeslot          { border: 1px solid  #00F000; }
.record_once              { border: 1px solid  #00F000; }
.record_channel           { border: 1px solid  #00F000; }
.record_always            { border: 1px solid  #00F000; }
.record_conflicting       { border: 1px solid  #FF0000; }
.record_duplicate         { border: 2px dashed #90B0A0; }
.record_old_duplicate     { border: 1px dashed #90B0A0; }
.record_suppressed        { border: 2px dotted #60B0B0; }
.record_override_suppress { border: 2px dotted #D03060; }
.record_override_record   { border: 2px solid  #00FF90; }


/* There should be exactly one class in the next section corresponding to
    each possible value of RecStatus_Types as defined in
    includes/programs.php */
.rec_class {
    background:  transparent;
    text-align:  center;
}
.rec_class.WillRecord     { font-weight: bold; }
.rec_class.Recording      { font-weight: bold; }
.Recording                { color: #00FF00; border-width: 2px }
.WillRecord               { color: #00FF00; }
.PreviousRecording        { color: #90B0A0; }
.CurrentRecording         { color: #90D0A0; }
.Repeat                   { color: #90B0A0; }
.EarlierShowing           { color: #90FFA0; }
.LaterShowing             { color: #90FFA0; }
.TooManyRecordings        { color: #60B0B0; }
.Cancelled                { color: #60B0B0; }
.Conflicting              { color: #FF0000; }
.LowDiskSpace             { color: #60B0B0; }
.TunerBusy                { color: #60B0B0; }
.ManualOverride           { color: #D03060; }
.Overlap                  { color: #FF0000; }

.cat_Action         { background-color: #906020; }
.cat_Adult          { background-color: #702020; }
.cat_Animals        { background-color: #609060; }
.cat_Art_Music      { background-color: #801060; }
.cat_Business       { background-color: #703010; }
.cat_Children       { background-color: #B00010; }
.cat_Comedy         { background-color: #006080; }
.cat_Crime_Mystery  { background-color: #105050; }
.cat_Documentary    { background-color: #504020; }
.cat_Drama          { background-color: #400060; }
.cat_Educational    { background-color: #606060; }
.cat_Food           { background-color: #208040; }
.cat_Game           { background-color: #701010; }
.cat_Health_Medical { background-color: #806060; }
.cat_History        { background-color: #807020; }
.cat_HowTo          { background-color: #A0A000; }
.cat_Horror         { background-color: #101040; }
.cat_Misc           { background-color: #403060; }
.cat_News           { background-color: #606040; }
.cat_Reality        { background-color: #304040; }
.cat_Romance        { background-color: #A02050; }
.cat_Science_Nature { background-color: #008050; }
.cat_SciFi_Fantasy  { background-color: #606090; }
.cat_Shopping       { background-color: #103010; }
.cat_Soaps          { background-color: #508080; }
.cat_Spiritual      { background-color: #804080; }
.cat_Sports         { background-color: #005030; }
.cat_Talk           { background-color: #203040; }
.cat_Travel         { background-color: #2060B0; }
.cat_War            { background-color: #B06050; }
.cat_Western        { background-color: #806040; }
.cat_Unknown        { background-color: #303030; }

.cat_movie          { background-color: #809090 !important; }
.cat_series         { }
.cat_tvshow         { }

/* deprecated show categories that need to be integreted into the above list via regex, or have new categories created for them */
.tv_Cine           { background-color: blue;            }
.tv_Classic        { background-color: gray;            }
.tv_Collectibles   { background-color: slategray;       }
.tv_Magazine       { background-color: mediumturquoise; }
.tv_Movies         { background-color: blue;            }
.tv_Serie          { background-color: darkorchid;      }
.tv_SHOW           { background-color: sienna;          }
.tv_Tiempo         { background-color: #33ff99;         }


