#AppPanel {
    display:                block !important;
    width:                  100%;
    color:                  #FFFFFF;
    background-color:       #000000;
    min-height:             380px;
}

body.landscape #AppPanel {
    min-height:             230px;
}

.appicon {
    float:                  left;
    width:                  72px;
    height:                 80px;
    padding-left:           4px;
    padding-right:          4px;
    padding-top:            20px;
    text-align:             center;
}

.appicon a {
    text-decoration:        none;
    color:                  white;
    font-family:            Helvetica;
    margin-left:            auto;
    margin-right:           auto;
}

.appicon a img {
    border:                 0px;
    height:                 60px;
    width:                  60px;
}
