<?php
/**
 * Initialization routines for the MythWeb Video Coverart module
 *
 * @url         $URL$
 * @date        $Date$
 * @version     $Revision$
 * @author      $Author$
 * @license     GPL
 *
 * @package     MythWeb
 * @subpackage  Video Coverart
 *
/**/
?>
