/*

    The following subclasses are used for list displays

*/

/* a class for the the cells containing recored programs in recorded_programs.php */
.recorded {    background-color: #dddddd; } /*#506080*/

/* a class for the the cells containing recorded programs in recorded_programs.php */
.scheduled   { background-color: #E5EEFF; !important; } /*hsl(220,100%,95%)*/
.conflict    { background-color: #FFE5E5; !important; } /*hsl(0,100%,95%)*/
.duplicate   { background-color: #E5FAFF; !important; } /*hsl(190,100%,95%)*/
.deactivated { background-color: #F6E5FF; !important; } /*hsl(280,100%,95%)*/

.record_weekly            { border: 2px solid  #00E400; }
.record_timeslot          { border: 2px solid  #00E400; }
.record_once              { border: 2px solid  #00E400; }
.record_channel           { border: 2px solid  #00E400; }
.record_always            { border: 2px solid  #00E400; }
.will_record              { border: 2px solid  #00E400; }
.record_conflicting       { border: 2px solid  #FF0000; }
.record_duplicate         { border: 2px dashed #90B0A0; }
.record_old_duplicate     { border: 1px dashed #90B0A0; }
.record_suppressed        { border: 2px dotted #60B0B0; }
.record_override_suppress { border: 2px dotted #D03060; }
.record_override_record   { border: 2px solid  #00FF90; }


/* There should be exactly one class in the next section corresponding to
    each possible value of RecStatus_Types as defined in
    includes/programs.php */
.rec_class {
    background:  transparent;
    text-align:  center;
    font-weight: bold;
}
.recording                { color: #00F000; }
.willRecord               { color: #00F000; }
.previousRecording        { color: #90B0A0; }
.currentRecording         { color: #00F000; }
.repeat                   { color: #90B0A0; }
.earlierShowing           { color: #90B0A0; }
.laterShowing             { color: #90B0A0; }
.tooManyRecordings        { color: #60B0B0; }
.cancelled                { color: #60B0B0; }
.conflicting              { color: #FF0000; }
.lowDiskSpace             { color: #60B0B0; }
.tunerBusy                { color: #60B0B0; }
.manualOverride           { color: #D03060; }
.overlap                  { color: #FF0000; }

.cat_Action         { background-color: #FBCEC4; } /*hsl(10,90%,88%)*/
.cat_Adult          { background-color: #F8CBB3; } /*hsl(21,85%,84%)*/
.cat_Animals        { background-color: #FBE2C4; } /*hsl(32,90%,88%)*/
.cat_Art_Music      { background-color: #F8E5B3; } /*hsl(43,85%,84%)*/
.cat_Business       { background-color: #FBF7C4; } /*hsl(55,90%,88%)*/
.cat_Children       { background-color: #F1F8B3; } /*hsl(66,85%,84%)*/
.cat_Comedy         { background-color: #ECFBC4; } /*hsl(77,90%,88%)*/
.cat_Crime_Mystery  { background-color: #D8F8B3; } /*hsl(88,85%,84%)*/
.cat_Documentary    { background-color: #D7FBC4; } /*hsl(100,90%,88%)*/
.cat_Drama          { background-color: #BDF8B3; } /*hsl(111,85%,84%)*/
.cat_Educational    { background-color: #C4FBC6; } /*hsl(122,90%,88%)*/
.cat_Food           { background-color: #B3F8C2; } /*hsl(133,85%,84%)*/
.cat_Game           { background-color: #C4FBDB; } /*hsl(145,90%,88%)*/
.cat_Health_Medical { background-color: #B3F8DD; } /*hsl(156,85%,84%)*/
.cat_History        { background-color: #C4FBF0; } /*hsl(167,90%,88%)*/
.cat_HowTo          { background-color: #B3F8F6; } /*hsl(178,85%,84%)*/
.cat_Horror         { background-color: #C4F2FB; } /*hsl(190,90%,88%)*/
.cat_Misc           { background-color: #B3E0F8; } /*hsl(201,85%,84%)*/
.cat_News           { background-color: #C4DEFB; } /*hsl(212,90%,88%)*/
.cat_Reality        { background-color: #B3C7F8; } /*hsl(223,85%,84%)*/
.cat_Romance        { background-color: #C4C9FB; } /*hsl(235,90%,88%)*/
.cat_Science_Nature { background-color: #BAB3F8; } /*hsl(246,85%,84%)*/
.cat_SciFi_Fantasy  { background-color: #D4C4FB; } /*hsl(257,90%,88%)*/
.cat_Shopping       { background-color: #D3B3F8; } /*hsl(268,85%,84%)*/
.cat_Soaps          { background-color: #E9C4FB; } /*hsl(280,90%,88%)*/
.cat_Spiritual      { background-color: #EEB3F8; } /*hsl(291,85%,84%)*/
.cat_Sports         { background-color: #FBC4FA; } /*hsl(302,90%,88%)*/
.cat_Talk           { background-color: #F8B3E9; } /*hsl(313,85%,84%)*/
.cat_Travel         { background-color: #FBC4E4; } /*hsl(325,90%,88%)*/
.cat_War            { background-color: #F8B3CF; } /*hsl(336,85%,84%)*/
.cat_Western        { background-color: #FBC4D0; } /*hsl(347,90%,88%)*/
.cat_Unknown        { background-color: #EDEDED; } /*hsl(0,0%,93%)*/

.cat_movie          { background-color: #F8B3B5; !important; } /*hsl(358,85%,84%)*/
.cat_series         { }
.cat_tvshow         { }

/* deprecated show categories that need to be integreted into the above list via regex, or have new categories created for them */
.tv_Cine           { background-color: blue;            }
.tv_Classic        { background-color: gray;            }
.tv_Collectibles   { background-color: slategray;       }
.tv_Magazine       { background-color: mediumturquoise; }
.tv_Movies         { background-color: blue;            }
.tv_Serie          { background-color: darkorchid;      }
.tv_SHOW           { background-color: sienna;          }
.tv_Tiempo         { background-color: #33ff99;         }


