/*
 *
 * The following subclasses are used for list displays that deal with tv
 * programming.
 *
/*/

/* some classes for various list displays */
    .list {    background-color: #DDDDDD }
    .list_separator {
        background-color: #CCCCCC;
        border-top:       4px;
        background-image: url(img/background-gradient-h.png);
        background-repeat: repeat-y;
        font-weight:        bold;
        color: #FFFFFF;
    }

/* The category legend, and its associated popup */
    #command_choices a {
        white-space: nowrap;
    }

    #category_legend_popup {
        position:         absolute;
        visibility:       hidden;
        top:              0px;
        left:             0px;
        padding:          2px;
        white-space:      nowrap;
    }
    #category_legend_popup table {
        font-size:        9pt;
    }

/* A class for the left-cleared <dl> lists used for program details popups */
    dl.details_list {
        font-size:  9pt;
        margin:     .125em;
        max-width:  41em;
    }
    dl.details_list dt {
        clear:       left;
        float:       left;
        width:       9em;
        padding-top: 3px;
        text-align:  right;
        color:       #EEEEEE;
        white-space: nowrap;
    }
    dl.details_list dd {
        padding-top: 3px;
        margin-left: 9.5em;
    }

/* Control the look of the text-based HDTV stamp throughout MythWeb */
    .hdtv_icon {
        font-size:   9pt;
        float:       right;
        color:       yellow;
        display:     inline;
        font-weight: bold;
    }

/* a class for the the cells containing recorded programs in recorded_programs.php */
.scheduled   { background-color: #CCDDDD; }
.conflict    { background-color: #806050; }
.duplicate   { background-color: #304050; }
.deactivated { background-color: #508060; }

.will_record              { border: 1px solid  #00F000 !important; }
.record_weekly            { border: 1px solid  #00F000 !important; }
.record_timeslot          { border: 1px solid  #00F000 !important; }
.record_once              { border: 1px solid  #00F000 !important; }
.record_channel           { border: 1px solid  #00F000 !important; }
.record_always            { border: 1px solid  #00F000 !important; }
.record_conflicting       { border: 1px solid  #FF0000 !important; }
.record_duplicate         { border: 2px dashed #90B0A0 !important; }
.record_old_duplicate     { border: 1px dashed #90B0A0 !important; }
.record_suppressed        { border: 2px dotted #60B0B0 !important; }
.record_override_suppress { border: 2px dotted #D03060 !important; }
.record_override_record   { border: 2px solid  #00FF90 !important; }


/* There should be exactly one class in the next section corresponding to
    each possible value of RecStatus_Types as defined in
    includes/programs.php */
.rec_class {
    background:  transparent;
    text-align:  center;
}
.rec_class.WillRecord     { font-weight: bold; }
.rec_class.Recording      { font-weight: bold; }
.Recording                { color: #00FF00; border-width: 2px }
.WillRecord               { color: #00FF00; }
.PreviousRecording        { color: #90B0A0; }
.CurrentRecording         { color: #90D0A0; }
.Repeat                   { color: #90B0A0; }
.EarlierShowing           { color: #90FFA0; }
.LaterShowing             { color: #90FFA0; }
.TooManyRecordings        { color: #60B0B0; }
.Cancelled                { color: #60B0B0; }
.Conflicting              { color: #FF0000; }
.LowDiskSpace             { color: #60B0B0; }
.TunerBusy                { color: #60B0B0; }
.ManualOverride           { color: #D03060; }
.Overlap                  { color: #FF0000; }

.cat_Action         { background-color: #FFCC66; }
.cat_Adult          { background-color: #FF9999; }
.cat_Animals        { background-color: #CCFF33; }
.cat_Art_Music      { background-color: #66FF66; }
.cat_Business       { background-color: #703010; }
.cat_Children       { background-color: #CC99CC; }
.cat_Comedy         { background-color: #FFCCCC; }
.cat_Crime_Mystery  { background-color: #105050; }
.cat_Documentary    { background-color: #CC9900; }
.cat_Drama          { background-color: #66CC99; }
.cat_Educational    { background-color: #CCCC99; }
.cat_Food           { background-color: #208040; }
.cat_Game           { background-color: #701010; }
.cat_Health_Medical { background-color: #806060; }
.cat_History        { background-color: #AAAAAA; }
.cat_HowTo          { background-color: #A0A000; }
.cat_Horror         { background-color: #101040; }
.cat_Misc           { background-color: #403060; }
.cat_News           { background-color: #CCFF99; }
.cat_Reality        { background-color: #304040; }
.cat_Romance        { background-color: #A02050; }
.cat_Science_Nature { background-color: #008050; }
.cat_SciFi_Fantasy  { background-color: #606090; }
.cat_Shopping       { background-color: #103010; }
.cat_Soaps          { background-color: #CCCC00; }
.cat_Spiritual      { background-color: #804080; }
.cat_Sports         { background-color: #99FF00; }
.cat_Talk           { background-color: #203040; }
.cat_Travel         { background-color: #2060B0; }
.cat_War            { background-color: #B06050; }
.cat_Western        { background-color: #806040; }
.cat_Unknown        { background-color: #FFFFFF; }

.cat_movie          { background-color: #CCCC00 !important; }
.cat_series         { }
.cat_tvshow         { }

/* deprecated show categories that need to be integreted into the above list via regex, or have new categories created for them */
.tv_Cine           { background-color: blue;            }
.tv_Classic        { background-color: gray;            }
.tv_Collectibles   { background-color: #708090;         }
.tv_Magazine       { background-color: #48D1CC;         }
.tv_Movies         { background-color: blue;            }
.tv_Serie          { background-color: #9932CC;         }
.tv_SHOW           { background-color: #A0522D;         }
.tv_Tiempo         { background-color: #33ff99;         }
