.weather_site {
    margin: 10px;
    border: 1px solid;
}

.weather_site h2 {
    font-weight: normal;
    margin:      0px;
    padding:     0px 0px 10px 0px;
}


.weather_site .host {
    float:         right;
    font-style:    italic;
    font-weight:   bold;
    margin:        10px;
}

.weather_site .location {
    font-size:    12pt;
    padding-left: .5em;
    padding-top:  0;
}

.weather_site .current_conditions {
    margin:  10px 20px;
    border:  1px solid gray;
    padding: 10px;
}

.weather_site .current_conditions .overview {
    float:      left;
    width:      15em;
    margin:     0px 1em;
    padding:    5px;
    /*border:     1px solid gray;*/
    text-align: center;
}
.weather_site .current_conditions .overview h3 {
    font-weight: normal;
    font-size:   12pt;
    margin:      0px;
    padding:     0px;
}
.weather_site .current_conditions .overview p.temp {
    font-size:  24pt;
    margin:     0px;
    padding:    0px;
}
.weather_site .current_conditions .overview p.temp sup {
    font-size: 10pt;
}

.weather_site .current_conditions table {
    line-height: 1.6em;
}
.weather_site .current_conditions table th {
    font-weight: normal;
    text-align:  left;
    width:       8em;
}
.weather_site .current_conditions table td {
}
.weather_site .current_conditions table td.temp {
}
.weather_site .current_conditions table td.temp sup {
    font-size: 7pt;
    position:  relative;
    top:       3px;
}

.weather_site .forecast {
    margin:  10px 20px;
    border:  1px solid gray;
    padding: 10px;
}
.weather_site .forecast .daily_forecast {
    float:      left;
    width:      15em;
    margin:     0px 0px 10px 10px;
    padding:    5px;
    border:     1px solid gray;
    text-align: center;
}
.weather_site .forecast .daily_forecast h3 {
    margin:      0px;
    padding:     0px;
    font-weight: normal;
    font-size:   12pt;
}
.weather_site .forecast .daily_forecast .temps {
    width:      80%;
    margin:     3px auto 0px auto;
}
.weather_site .forecast .daily_forecast .temps div {
    float:      left;
    width:      50%;
    text-align: center;
}
.weather_site .forecast .daily_forecast .temps p {
    margin:  0px;
    padding: 0px;
}
.weather_site .forecast .daily_forecast .temps p.temp {
    margin:     0px 2em;
    font-size:  18pt;
    margin:     0px;
    padding:    0px;
}
.weather_site .forecast .daily_forecast .temps p.temp sup {
    font-size: 9pt;
}

.weather_site .radar {
    margin:  10px 20px;
    border:  1px solid gray;
    padding: 10px;
}

.weather_site .last_updated {
    text-align: right;
    margin:     10px;
}
