<?php
/**
 *
 * @url         $URL: http://svn.mythtv.org/svn/branches/release-0-23-fixes/mythplugins/mythweb/modules/remote/tmpl/iPod/welcome.php $
 * @date        $Date: 2009-08-09 09:05:07 +0300 (Sun, 09 Aug 2009) $
 * @version     $Revision: 21182 $
 * @author      $Author: kormoc $
 * @license     GPL
 *
 * @package     MythWeb
 *
/**/

$Module_Name = 'Remote';
$Module_Icon = skin_url.'img/Icons/remote';
$Module_URL  = '#';
