h3 {
	margin:                             0;
	padding-top:                        10px;
	padding-right:                      10px;
	padding-bottom:                     10px;
	padding-left:                       10px;
	font-size:                          17px;
	font-family:                        Helvetica;
	font-weight:                        bold;
	color:                              rgb(76,86,108);
    max-width:                          305px;
    white-space:                        nowrap;
    overflow:                           hidden;
    text-overflow:                      ellipsis;
}

body.landscape h3 {
    max-width:                          465px;
}

.ListPanel {
    background:                         transparent;
    margin-left:                        5px;
    margin-right:                       5px;
    margin-top:                         5px;
    margin-bottom:                      5px;
    padding:                            0px;
}

.ListPanel > li {
    background-color:                   #FFFFFF;
    border-left:                        1px solid rgb(171,173,176);
    border-right:                       1px solid rgb(171,173,176);
    border-bottom:                      1px solid rgb(171,173,176);
    font-size:                          20px;
    font-weight:                        bold;
    list-style-image:                   none;
    list-style-position:                outside;
    list-style-type:                    none;
    max-height:                         50px;
}

.ListPanel > li.long {
    max-height:                         1000px !important;
    white-space:                        normal !important;
    font-size:                          12px;
}

.ListPanel > li.text
{
    max-height:                         40px;
    white-space:                        nowrap;
    overflow:                           hidden;
    text-overflow:                      ellipsis;
    padding-bottom:                     8px;
    padding-left:                       8px;
    padding-right:                      32px;
    padding-top:                        8px;
}

.ListPanel > li.small {
    font-size:                          12px;
    max-height:                         28px;
    padding-top:                        0px;
    padding-bottom:                     0px;
}

.ListPanel > li.small.text {
    padding-top:                        4px;
    padding-bottom:                     4px;
}

.ListPanel > li:first-child {
    -webkit-border-top-left-radius:     8px;
    -webkit-border-top-right-radius:    8px;
    border-top:                         1px solid rgb(171,173,176);
}

.ListPanel > li:last-child {
    -webkit-border-bottom-left-radius:  8px;
    -webkit-border-bottom-right-radius: 8px;
    border-bottom:                      1px solid rgb(171,173,176);
}

.ListPanel > li > a {
    background:                         #FFFFFF url('img/chevron.png') no-repeat scroll right center;
    background-position:                290px center;
    color:                              inherit;
    display:                            block;
    text-decoration:                    none;
    max-height:                         inherit;
    white-space:                        nowrap;
    overflow:                           hidden;
    text-overflow:                      ellipsis;
    padding-bottom:                     8px;
    padding-left:                       8px;
    padding-right:                      32px;
    padding-top:                        8px;
}

.ListPanel > li.small > a {
    padding-top:                        4px;
    padding-bottom:                     4px;
}

body.landscape .ListPanel > li > a {
    background-position:                450px center;
}

.ListPanel > li > a.nochevron {
    background:                         #FFFFFF no-repeat scroll right center;
}

.ListPanel > li > a.nochevron:active,
.ListPanel > li > a.nochevron:focus,
.ListPanel > li > a.nochevron:hover
{
    background-image:                   url('img/item_background_touched.png');
    background-repeat:                  repeat-x;
    background-attachment:              scroll;
    background-position:                top;
}

.ListPanel > li > a:active,
.ListPanel > li > a:focus,
.ListPanel > li > a:hover
{
    color:                              white;
    background-color:                   rgb(1,95,231);
    background-image:                   url('img/chevron_touched.png'), url('img/item_background_touched.png');
    background-repeat:                  no-repeat, repeat-x;
    background-attachment:              scroll, scroll;
    background-position:                290px center, top;
}

body.landscape .ListPanel > li > a:active,
body.landscape .ListPanel > li > a:focus,
body.landscape .ListPanel > li > a:hover
{
    background-position:                450px center, top;
}

.ListPanel > li:first-child > a {
    -webkit-border-top-left-radius:     8px;
    -webkit-border-top-right-radius:    8px;
    border-top:                         1px solid rgb(171,173,176);
}

.ListPanel > li:last-child > a {
    -webkit-border-bottom-left-radius:  8px;
    -webkit-border-bottom-right-radius: 8px;
    border-bottom:                      1px solid rgb(171,173,176);
}

.right {
    float:                              right;
}
