/*

    The following subclasses are used for list displays

*/

/* a class for the the cells containing recored programs in recorded_programs.php */
.recorded {    background-color: #444444; } /*#506080*/

/* a class for the the cells containing recorded programs in recorded_programs.php */
.scheduled   { background-color: #506090; }
.conflict    { background-color: #806050; }
.duplicate   { background-color: #304050; }
.deactivated { background-color: #508060; }

.record_weekly            { border: 1px solid  #00F000; }
.record_timeslot          { border: 1px solid  #00F000; }
.record_once              { border: 1px solid  #00F000; }
.record_channel           { border: 1px solid  #00F000; }
.record_always            { border: 1px solid  #00F000; }
.will_record              { border: 1px solid  #00F000; }
.record_conflicting       { border: 1px solid  #FF0000; }
.record_duplicate         { border: 2px dashed #90B0A0; }
.record_old_duplicate     { border: 1px dashed #90B0A0; }
.record_suppressed        { border: 2px dotted #60B0B0; }
.record_override_suppress { border: 2px dotted #D03060; }
.record_override_record   { border: 2px solid  #00FF90; }


/* There should be exactly one class in the next section corresponding to
    each possible value of RecStatus_Types as defined in
    includes/programs.php */
.rec_class {
    background:  transparent;
    text-align:  center;
    font-weight: bold;
}
.recording                { color: #00F000; }
.willRecord               { color: #00F000; }
.previousRecording        { color: #90B0A0; }
.currentRecording         { color: #00F000; }
.repeat                   { color: #90B0A0; }
.earlierShowing           { color: #90B0A0; }
.laterShowing             { color: #90B0A0; }
.tooManyRecordings        { color: #60B0B0; }
.cancelled                { color: #60B0B0; }
.conflicting              { color: #FF0000; }
.lowDiskSpace             { color: #60B0B0; }
.tunerBusy                { color: #60B0B0; }
.manualOverride           { color: #D03060; }
.overlap                  { color: #FF0000; }

.cat_Action         { background-color: #6B1E0F; } /*hsl(10,75%,24%)*/
.cat_Adult          { background-color: #A34614; } /*hsl(21,78%,36%)*/
.cat_Animals        { background-color: #6B400F; } /*hsl(32,75%,24%)*/
.cat_Art_Music      { background-color: #A37A14; } /*hsl(43,78%,36%)*/
.cat_Business       { background-color: #6B630F; } /*hsl(55,75%,24%)*/
.cat_Children       { background-color: #95A314; } /*hsl(66,78%,36%)*/
.cat_Comedy         { background-color: #516B0F; } /*hsl(77,75%,24%)*/
.cat_Crime_Mystery  { background-color: #60A314; } /*hsl(88,78%,36%)*/
.cat_Documentary    { background-color: #2D6B0F; } /*hsl(100,75%,24%)*/
.cat_Drama          { background-color: #29A314; } /*hsl(111,78%,36%)*/
.cat_Educational    { background-color: #0F6B12; } /*hsl(122,75%,24%)*/
.cat_Food           { background-color: #14A333; } /*hsl(133,78%,36%)*/
.cat_Game           { background-color: #0F6B35; } /*hsl(145,75%,24%)*/
.cat_Health_Medical { background-color: #14A36A; } /*hsl(156,78%,36%)*/
.cat_History        { background-color: #0F6B57; } /*hsl(167,75%,24%)*/
.cat_HowTo          { background-color: #14A39E; } /*hsl(178,78%,36%)*/
.cat_Horror         { background-color: #0F5B6B; } /*hsl(190,75%,24%)*/
.cat_Misc           { background-color: #1471A3; } /*hsl(201,78%,36%)*/
.cat_News           { background-color: #0F3A6B; } /*hsl(212,75%,24%)*/
.cat_Reality        { background-color: #143CA3; } /*hsl(223,78%,36%)*/
.cat_Romance        { background-color: #0F166B; } /*hsl(235,75%,24%)*/
.cat_Science_Nature { background-color: #2214A3; } /*hsl(246,78%,36%)*/
.cat_SciFi_Fantasy  { background-color: #290F6B; } /*hsl(257,75%,24%)*/
.cat_Shopping       { background-color: #5714A3; } /*hsl(268,78%,36%)*/
.cat_Soaps          { background-color: #4C0F6B; } /*hsl(280,75%,24%)*/
.cat_Spiritual      { background-color: #8D14A3; } /*hsl(291,78%,36%)*/
.cat_Sports         { background-color: #6B0F68; } /*hsl(302,75%,24%)*/
.cat_Talk           { background-color: #A31484; } /*hsl(313,78%,36%)*/
.cat_Travel         { background-color: #6B0F44; } /*hsl(325,75%,24%)*/
.cat_War            { background-color: #A3144D; } /*hsl(336,78%,36%)*/
.cat_Western        { background-color: #6B0F23; } /*hsl(347,75%,24%)*/
.cat_Unknown        { background-color: #303030; } /*hsl(0,0%,19%)*/

.cat_movie          { background-color: #A31418; !important; } /*hsl(358,78%,36%)*/
.cat_series         { }
.cat_tvshow         { }

/* deprecated show categories that need to be integreted into the above list via regex, or have new categories created for them */
.tv_Cine           { background-color: blue;            }
.tv_Classic        { background-color: gray;            }
.tv_Collectibles   { background-color: slategray;       }
.tv_Magazine       { background-color: mediumturquoise; }
.tv_Movies         { background-color: blue;            }
.tv_Serie          { background-color: darkorchid;      }
.tv_SHOW           { background-color: sienna;          }
.tv_Tiempo         { background-color: #33ff99;         }


