void strings_null() {
    ThemeUI::tr("A high quality profile giving approx. 1 hour of video on a single layer DVD");
    ThemeUI::tr("A standard play profile giving approx. 2 hour of video on a single layer DVD");
    ThemeUI::tr("A long play profile giving approx. 4 hour of video on a single layer DVD");
    ThemeUI::tr("A extended play profile giving approx. 6 hour of video on a single layer DVD");
    ThemeUI::tr("Select Destination");
    ThemeUI::tr("Choose where you would like your files archived.");
    ThemeUI::tr("Output Type:");
    ThemeUI::tr("Destination:");
    ThemeUI::tr("Free Space:");
    ThemeUI::tr("Click here to find an output location...");
    ThemeUI::tr("Make ISO Image");
    ThemeUI::tr("Burn to DVD");
    ThemeUI::tr("Erase DVD-RW before burning");
    ThemeUI::tr("Cancel");
    ThemeUI::tr("Previous");
    ThemeUI::tr("Next");
    ThemeUI::tr("Filter:");
    ThemeUI::tr("OK");
    ThemeUI::tr("File Finder");
    ThemeUI::tr("Select the file you wish to use.");
    ThemeUI::tr("Back");
    ThemeUI::tr("Home");
    ThemeUI::tr("No videos available");
    ThemeUI::tr("Log Viewer");
    ThemeUI::tr("See logs from your archive runs.");
    ThemeUI::tr("Update");
    ThemeUI::tr("Exit");
    ThemeUI::tr("description goes here.");
    ThemeUI::tr("0.00Gb");
    ThemeUI::tr("Force Overwrite of DVD-RW Media");
    ThemeUI::tr("Select Recordings");
    ThemeUI::tr("Show Recordings");
    ThemeUI::tr("Select Videos");
    ThemeUI::tr("Video Category");
    ThemeUI::tr("title goes here");
    ThemeUI::tr("x.xx Gb");
    ThemeUI::tr("PL:");
    ThemeUI::tr("1");
    ThemeUI::tr("Change Encoding Profile");
    ThemeUI::tr("12.34GB");
    ThemeUI::tr("DVD Menu Theme");
    ThemeUI::tr("Choose the appearance of your DVD.");
    ThemeUI::tr("Theme:");
    ThemeUI::tr("Intro");
    ThemeUI::tr("Main Menu");
    ThemeUI::tr("Chapter Menu");
    ThemeUI::tr("Details");
    ThemeUI::tr("Select Archive Items");
    ThemeUI::tr("Select the recordings and videos you wish to save.");
    ThemeUI::tr("No files are selected for archive");
    ThemeUI::tr("Add Recording");
    ThemeUI::tr("Add Video");
    ThemeUI::tr("Add File");
    ThemeUI::tr("Archive Item Details");
    ThemeUI::tr("Title:");
    ThemeUI::tr("Subtitle:");
    ThemeUI::tr("Start Date:");
    ThemeUI::tr("Time:");
    ThemeUI::tr("Description:");
    ThemeUI::tr("Thumb Image Selector");
    ThemeUI::tr("Current Position");
    ThemeUI::tr("0:00:00.00");
    ThemeUI::tr("Seek Amount");
    ThemeUI::tr("Frame");
    ThemeUI::tr("Save");
    ThemeUI::tr("Select a theme");
    ThemeUI::tr("xxxxx mb");
    ThemeUI::tr("0 mb");
    ThemeUI::tr("Find");
    ThemeUI::tr("Prev");
    ThemeUI::tr("sep 13, 2004 11:00 pm (1h 15m)");
    ThemeUI::tr("File Finder To Import");
    ThemeUI::tr("Start Time:");
    ThemeUI::tr("Select Associated Channel");
    ThemeUI::tr("Archived Channel");
    ThemeUI::tr("Chan. ID:");
    ThemeUI::tr("Chan. No:");
    ThemeUI::tr("Callsign:");
    ThemeUI::tr("Name:");
    ThemeUI::tr("Local Channel");
    ThemeUI::tr("Search Channel");
    ThemeUI::tr("Search Callsign");
    ThemeUI::tr("Search Name");
    ThemeUI::tr("Finish");
    ThemeUI::tr("Add video");
    ThemeUI::tr("Create DVD");
    ThemeUI::tr("Create Archive");
    ThemeUI::tr("Encode Video File");
    ThemeUI::tr("Import Archive");
    ThemeUI::tr("Archive Utilities");
    ThemeUI::tr("Show Log Viewer");
    ThemeUI::tr("Play Created DVD");
    ThemeUI::tr("Burn DVD");
}
