void strings_null() {
    ThemeUI::tr("Edit RSS News Feeds");
    ThemeUI::tr("You can edit the name, URL, and icon path for the selected RSS Feed on this screen.  You can also designate the item as a podcast.\n\nWhen you have finished editing, select \"OK\" or \"Cancel\" to continue.");
    ThemeUI::tr("Name:");
    ThemeUI::tr("URL:");
    ThemeUI::tr("Icon:");
    ThemeUI::tr("Podcast:");
    ThemeUI::tr("Genre:");
    ThemeUI::tr("Channel:");
    ThemeUI::tr("Cancel");
    ThemeUI::tr("OK");
    ThemeUI::tr("News Subscriptions");
    ThemeUI::tr("Choose the news sites you would like to read.");
    ThemeUI::tr("Edit RSS News Feed");
    ThemeUI::tr("Ok");
    ThemeUI::tr("Contexts");
    ThemeUI::tr("Actions");
    ThemeUI::tr("RSS Feeds");
    ThemeUI::tr("MythNews Configuration");
    ThemeUI::tr("Minutes");
    ThemeUI::tr("News Feeds");
}
