void strings_null() {
    ThemeUI::tr("Select a Title to Rip");
    ThemeUI::tr("Select the parts of the DVD you wish to copy.");
    ThemeUI::tr("Film Name:");
    ThemeUI::tr("Track Length:");
    ThemeUI::tr("0:00:00");
    ThemeUI::tr("Rip Quality:");
    ThemeUI::tr("Audio Track:");
    ThemeUI::tr("Subtitle:");
    ThemeUI::tr("Rip AC-3 Audio");
    ThemeUI::tr("Preview this title");
    ThemeUI::tr("Rip This Track");
    ThemeUI::tr("Title 1 of 1");
    ThemeUI::tr("Previous Title");
    ThemeUI::tr("Next Title");
    ThemeUI::tr("Begin Ripping");
    ThemeUI::tr("No Current Jobs");
    ThemeUI::tr("Select a Track to Rip");
    ThemeUI::tr("Name of Film:");
    ThemeUI::tr("Selected track length:");
    ThemeUI::tr("Check this box to select this track:");
    ThemeUI::tr("Check to rip AC-3 audio:");
    ThemeUI::tr("Subtitles:");
    ThemeUI::tr("Press to view excerpt:");
    ThemeUI::tr("Previous");
    ThemeUI::tr("Next");
    ThemeUI::tr("Rip/Transcode");
    ThemeUI::tr("Job 1 of 1");
    ThemeUI::tr("Length:");
    ThemeUI::tr("Select:");
    ThemeUI::tr("Name:");
    ThemeUI::tr("Quality:");
    ThemeUI::tr("AC3 Audio:");
    ThemeUI::tr("View:");
    ThemeUI::tr("Cast");
    ThemeUI::tr("Plot");
    ThemeUI::tr("OK");
    ThemeUI::tr("Season/Episode: %1");
    ThemeUI::tr("Directed by %1");
    ThemeUI::tr("Starring: %1");
    ThemeUI::tr("Parental Level 0");
    ThemeUI::tr("Parental Level 1");
    ThemeUI::tr("Parental Level 2");
    ThemeUI::tr("Parental Level 3");
    ThemeUI::tr("Parental Level 4");
    ThemeUI::tr("Add New File Types");
    ThemeUI::tr("On this screen, you can add new extensions to be recognized by the Video Plugin.  If a scan does not add your files, create a new extension below by clicking the \"New\" button.\n\nIf you select \"Use Default Player,\" the player command set for the extension will be ignored.  \"Ignore This File Type\" prevents the items from appearing in a scan.");
    ThemeUI::tr("Extension:");
    ThemeUI::tr("Command:");
    ThemeUI::tr("Use Default Player");
    ThemeUI::tr("Ignore This File Type");
    ThemeUI::tr("New");
    ThemeUI::tr("Delete");
    ThemeUI::tr("Edit Metadata");
    ThemeUI::tr("Modify your film's metadata.");
    ThemeUI::tr("Title:");
    ThemeUI::tr("Rating:");
    ThemeUI::tr("Director:");
    ThemeUI::tr("TMDB Number:");
    ThemeUI::tr("Plot:");
    ThemeUI::tr("Cover Art:");
    ThemeUI::tr("Screenshot:");
    ThemeUI::tr("Banner:");
    ThemeUI::tr("Fanart:");
    ThemeUI::tr("Trailer:");
    ThemeUI::tr("Homepage:");
    ThemeUI::tr("Custom Player:");
    ThemeUI::tr("Film has been Watched");
    ThemeUI::tr("Film is Browsable");
    ThemeUI::tr("Season:");
    ThemeUI::tr("Episode:");
    ThemeUI::tr("Year:");
    ThemeUI::tr("User Rating:");
    ThemeUI::tr("Parental Level:");
    ThemeUI::tr("Category:");
    ThemeUI::tr("Next File:");
    ThemeUI::tr("Click here to search existing coverart...");
    ThemeUI::tr("Click here to search existing screenshots...");
    ThemeUI::tr("Click here to search existing banners...");
    ThemeUI::tr("Click here to search existing fanart...");
    ThemeUI::tr("Click here to search existing trailers...");
    ThemeUI::tr("Filter Videos");
    ThemeUI::tr("Genre:");
    ThemeUI::tr("Country:");
    ThemeUI::tr("Actor/Actress:");
    ThemeUI::tr("Runtime:");
    ThemeUI::tr("Browse:");
    ThemeUI::tr("TMDB/TVDB #:");
    ThemeUI::tr("Coverart:");
    ThemeUI::tr("Sort By:");
    ThemeUI::tr("Watched:");
    ThemeUI::tr("Text Filter:");
    ThemeUI::tr("Result");
    ThemeUI::tr("Save");
    ThemeUI::tr("This item was released on %1.");
    ThemeUI::tr("It was added to your collection on %1.");
    ThemeUI::tr("Cancel");
    ThemeUI::tr("Play");
    ThemeUI::tr("Video Player Settings");
    ThemeUI::tr("Default:");
    ThemeUI::tr("DVD Player:");
    ThemeUI::tr("DVD Drive:");
    ThemeUI::tr("VCD Player:");
    ThemeUI::tr("VCD Drive:");
    ThemeUI::tr("Enable Alternate Player");
    ThemeUI::tr("Alt. Player:");
    ThemeUI::tr("Video Metadata Settings");
    ThemeUI::tr("Movie Grab:");
    ThemeUI::tr("TV Grab:");
    ThemeUI::tr("Display unknown file types");
    ThemeUI::tr("Enable metadata in file browse mode");
    ThemeUI::tr("Enable Random Trailer Playback");
    ThemeUI::tr("Trailer #:");
    ThemeUI::tr("No videos in library, or no files found.  If you have configured a video directory, press \"M\" (or the MENU key) and select \"Scan for Changes.\"");
    ThemeUI::tr("Rated:");
    ThemeUI::tr("Browsable:");
    ThemeUI::tr("Filename:");
    ThemeUI::tr("Cover File:");
    ThemeUI::tr("Video Num.:");
    ThemeUI::tr("Directed by:");
    ThemeUI::tr("Use default player:");
    ThemeUI::tr("Ignore:");
    ThemeUI::tr("TMDB:");
    ThemeUI::tr("Parental Lvl:");
    ThemeUI::tr("Browseable");
    ThemeUI::tr("Watched");
    ThemeUI::tr("Player Command:");
    ThemeUI::tr("/path/to/the/thing.jpg");
    ThemeUI::tr("Done");
    ThemeUI::tr("Category :");
    ThemeUI::tr("Genre :");
    ThemeUI::tr("Country :");
    ThemeUI::tr("Actor/Actress :");
    ThemeUI::tr("Year :");
    ThemeUI::tr("Runtime :");
    ThemeUI::tr("User Rating :");
    ThemeUI::tr("Browse :");
    ThemeUI::tr("InetRef :");
    ThemeUI::tr("Cover File :");
    ThemeUI::tr("Sort by :");
    ThemeUI::tr("Watched :");
    ThemeUI::tr("Text Filter :");
    ThemeUI::tr("Seas/Ep:");
    ThemeUI::tr("Lowest");
    ThemeUI::tr("Low");
    ThemeUI::tr("Medium");
    ThemeUI::tr("High");
    ThemeUI::tr("SELECT: Edit");
    ThemeUI::tr("INFO - Action Menu");
    ThemeUI::tr("Directed by");
    ThemeUI::tr("Runtime");
    ThemeUI::tr("Year");
    ThemeUI::tr("User Rating");
    ThemeUI::tr("Video Gallery");
    ThemeUI::tr("Parental Level");
    ThemeUI::tr("MythVideo File Associations");
    ThemeUI::tr("Filters video list");
    ThemeUI::tr("Background:");
    ThemeUI::tr("Edit Video Information");
    ThemeUI::tr("Parental Control:");
    ThemeUI::tr("File to Always Play Next:");
    ThemeUI::tr("Include while Browsing:");
    ThemeUI::tr("Item has been watched:");
    ThemeUI::tr("Unique Player Command:");
    ThemeUI::tr("Video List Filters");
    ThemeUI::tr("Save as default");
    ThemeUI::tr("Player Settings");
    ThemeUI::tr("Default Player:");
    ThemeUI::tr("Enable Alternate Player:");
    ThemeUI::tr("Alternate Player:");
    ThemeUI::tr("Movie Grabber:");
    ThemeUI::tr("TV Grabber:");
    ThemeUI::tr("Trailers to Play:");
    ThemeUI::tr("General Settings");
    ThemeUI::tr("File Paths and general options");
    ThemeUI::tr("Player Applications and DVD/VCD Drives");
    ThemeUI::tr("Metadata Settings");
    ThemeUI::tr("Grabber Setup and display preferences");
    ThemeUI::tr("File Types");
    ThemeUI::tr("Associate new file extensions");
    ThemeUI::tr("Rip Settings");
    ThemeUI::tr("DVD Rip and Transcode options");
    ThemeUI::tr("Browse Videos");
    ThemeUI::tr("Video List");
    ThemeUI::tr("Video Manager");
    ThemeUI::tr("Play DVD");
    ThemeUI::tr("Play VCD");
    ThemeUI::tr("Rip DVD");
    ThemeUI::tr("Eject media");
    ThemeUI::tr("Videos Settings");
    ThemeUI::tr("Configure playback and DVD ripping");
}
